/* Bipartite.java
 * =========================================================================
 * This file is part of the GrInvIn project - http://www.grinvin.org
 * 
 * Copyright (C) 2005-2008 Universiteit Gent
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * A copy of the GNU General Public License can be found in the file
 * LICENSE.txt provided with the source distribution of this program (see
 * the META-INF directory in the source jar). This license can also be
 * found on the GNU website at http://www.gnu.org/licenses/gpl.html.
 * 
 * If you did not receive a copy of the GNU General Public License along
 * with this program, contact the lead developer, or write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

package org.grinvin.invariants.computers.booleaninvariants;

import java.util.LinkedList;
import java.util.ArrayList;
import java.util.List;
import java.util.Queue;

import org.grinvin.graphs.GraphBundleView;
import org.grinvin.invariants.InvariantValue;
import org.grinvin.invariants.computers.AbstractInvariantComputer;
import org.grinvin.invariants.values.BooleanValue;

/**
 * Computes whether the graph is bipartite.
 */
public class Bipartite extends AbstractInvariantComputer {

    public String getInvariantId() {
        return "org.grinvin.invariants.Bipartite";
    }

    public InvariantValue compute(GraphBundleView bundle) {
        int[][] adjlist = bundle.adjacencyList();
        List<Integer> uVertices = new ArrayList<Integer>();
        return new BooleanValue(isBipartite(adjlist, uVertices), this);
    }

    /**
     * This method test if the given graph is bipartite using BFS.
     * @return true if given graph is bipartite
     */
    private boolean isBipartite(int[][] adjlist, List<Integer> uVertices) {
        int[] partition = new int[adjlist.length];
        for(int i = 0; i < partition.length; i++)
            partition[i] = -1;
        for(int v = 0; v < adjlist.length; v++) {
            if(partition[v] == -1) {
                partition[v] = 0;
                uVertices.add(v);
                Queue<Integer> q = new LinkedList<Integer>();
                q.offer(new Integer(v));
                while(!q.isEmpty()) {
                    int vertex = q.poll().intValue();
                    for(int i = 0; i < adjlist[vertex].length; i++) {
                        int neighbour = adjlist[vertex][i];
                        if(partition[vertex] == partition[neighbour])
                            return false;
                        else {
                            if(partition[neighbour] == -1) {
                                if(partition[vertex] == 0) {
                                    partition[neighbour] = 1;
                                }
                                else {
                                    partition[neighbour] = 0;
                                    uVertices.add(neighbour);
                                }
                                q.offer(new Integer(neighbour));
                            }
                        }
                    }
                }
            }
        }
        return true;
    }
}
